__title__ = 'mybar'
__description__ = "Take your status bar to the next level."
__description__ = "Marshall your Fields to victory."
__url__ = "https://github.com/derek-yeetr/mybar"
__version__ = '0.1.0'
__author__ = "derek-yeetr"
__license__ = 'MIT'
__copyright__ = 'Copyright (c) 2022 derek-yeetr'

