"""
mybar
~~~~~

An asynchronous status bar with an intuitive, highly customizable API.

:copyright: (c) 2022 by derek-yeetr.
:license: MIT, see LICENSE for more details.
"""

__title__ = 'mybar'
__description__ = "An async status bar with a highly customizable API."
__url__ = "https://github.com/derek-yeetr/mybar"
__version__ = '0.2'
__author__ = "derek-yeetr"
__license__ = 'MIT'
__copyright__ = 'Copyright (c) 2022 derek-yeetr'


from . import field_funcs
from . import setups
from . import utils
from .base import *
from .errors import *

